//--------------------------------------------------------------------------------------------------
//--------------------------------------Bibliothèques-----------------------------------------------
//--------------------------------------------------------------------------------------------------

#include <MKRNB.h> // connexion 4G
#include <ThingSpeak.h> // fonctions ThingSpeak
#include <OneWire.h> // communication capteur T
#include <SPI.h> // communication carte SD
#include <SD.h> // communication carte SD
#include <RTCZero.h> // horloge

//--------------------------------------------------------------------------------------------------
//--------------------------------Initialisation des capteurs---------------------------------------
//--------------------------------------------------------------------------------------------------

const char PINNUMBER[] = "0000"; // pin carte SIM et APN ThingsMobile (A CHANGER !)
const char APN[] = "TM";

NBClient client; // initialisent les librairies du NB1500
GPRS gprs;
NB nbAccess;

unsigned long myChannelNumber = 0000000; // numero de channel ThingSpeak (A CHANGER !)
const char* myWriteAPIKey = "XXXX0000"; // clé API ThingSpeak

#define pin_T 2 // pin sur lequel est branchee le DS18S20 (digital)
#define pin_pH A1  // pin sur lequel est branchee le pH-metre
#define pin_EC A2    // pin sur lequel est branchee la sonde EC
#define pin_NTU A3  // pin sur lequel est branchee la sonde turbidite
#define pin_DO A4 // pin sur lequel est branchee la sonde oxygene dissous

#define VREF 5000.0    // tension de reference pour la conversion mesure analogique -> volts
#define SCOUNT 30      // somme des points de mesure (requis pour mesure EC)

OneWire ds(pin_T);  // initialisation du DS18S20

const int chipSelect = 4; // pin carte SD (pour le shield SD MKR)

RTCZero rtc; // initialise la librairie de l'horloge RTC

bool matched = true; // booleen alarme 

//--------------------------------------------------------------------------------------------------
//---------------------------------------Fonction SETUP---------------------------------------------
//--------------------------------------------------------------------------------------------------

void setup() {
  
  Serial.begin(9600); // freq de communication série

  delay(1000);
  
  while (!Serial) {
    ; // attente de connexion au port serie, necessaire pour arduino MKR
  }

  if (!SD.begin(chipSelect)) { // initialisation de la carte SD
    Serial.println("Pas de carte SD détectée !");
  while (1);
  }
  
  Serial.println("Carte SD OK.");

  delay(1000);

  Serial.print("Initialisation de l'horloge... "); 

  rtc.begin(); // initialisation de l'horloge 
  
  setTimeFromCompile(); // reglage heure et date d'après console
  setDateFromCompile();

  Serial.println("Horloge OK.");  

  File dataFile = SD.open("datalog.txt", FILE_WRITE); // ouvre le fichier texte datalog.txt

  String titleString = ""; 

  titleString = String("Horloge;") + "\t" + String("Temp(C);") + "\t" + String("pH;") + "\t" + String("EC(mS/cm);") + "\t" + String("Turbidite(NTU);")+ "\t" + String("DO(ug/L);");

  if (dataFile) {
   dataFile.println(titleString); // ecrit l'en-tête du fichier csv
   dataFile.close();
   Serial.println("Fichier datalog.txt OK.");
   }
  else {
    Serial.println("Erreur ouverture datalog.txt");
  }

  Serial.println("Sonde SEB OK !"); // début de l'expérience, plus de communication série (port coupé par Arduino MKR après mise en veille)
}

void loop() {

  if (matched){ // boucle if s'exécutant si l'alarme est déclenchée (matched == true)

    matched = false; // retire l'alarme

    int H = rtc.getHours(); // horloge
    int Mn = rtc.getMinutes();

    float T = getTemp(); // mesure temperature
    float pH = getpH(); // mesure pH
    float EC = getEC(T); // mesure EC
    float NTU = getNTU(); // mesure turbidite
    float DO = getDO(T); // mesure DO

    String dataString = "";

    dataString += String(H) + String(":") + String(Mn) + String(";") + "\t" + String(T) + String(";") + "\t" + String(pH) + String(";") + "\t" + String(EC) + String(";") + "\t" + String(NTU) + String(";") + "\t" + String(DO) + String(";");  
  
    File dataFile = SD.open("datalog.txt", FILE_WRITE); // ouverture en mode écriture du fichier sur la carte SD

    if (dataFile) {
      dataFile.println(dataString); // ecriture des données
      dataFile.close();
    }

    boolean connected = false; // etat de la connexion 4G

    while (!connected) {
      if ((nbAccess.begin(PINNUMBER,APN) == NB_READY) && // se connecte au réseau et attache le GPRS
        (gprs.attachGPRS() == GPRS_READY)) {
        connected = true;
      } 
      else {
        delay(1000);
      }
    }

    ThingSpeak.begin(client); // initialise la connexion avec ThingSpeak

    delay(1000);
  
    ThingSpeak.setField(1, T); // def des champs ThingSpeak
    ThingSpeak.setField(2, pH);
    ThingSpeak.setField(3, EC); 
    ThingSpeak.setField(4, NTU);
    ThingSpeak.setField(5, DO); 
     
    ThingSpeak.writeFields(myChannelNumber, myWriteAPIKey); // transmet les données en ligne

    delay(240000); // attend 4 mn (timeout du modem)

    nbAccess.shutdown(); // ferme la connexion

    delay(240000); // attend 4 mn (timeout du modem)

    int alarmMinutes = rtc.getMinutes(); // réglage de l'heure de l'alarme
    int alarmHours = rtc.getHours();
    
    alarmMinutes += 5; // alarme toutes les xxx mn
    //alarmHours += 1; // alarme toutes les xxx h
    
    if (alarmMinutes > 59) {
      alarmMinutes -= 60;
      alarmHours += 1;
    }

    if(alarmHours > 23) {
      alarmHours -= 24;     
    }

    rtc.setAlarmTime(alarmHours, alarmMinutes, 00);
    rtc.enableAlarm(rtc.MATCH_HHMMSS); // Allume l'alarme
    rtc.attachInterrupt(alarmMatch); // change le booleen matched en "true" lorsque l'alarme se declenche
    rtc.standbyMode();    // veille jusqu'a la prochaine alarme     
  }  
}



//--------------------------------------------------------------------------------------------------
//---------------------------------Fonction Mesure Temperature--------------------------------------
//--------------------------------------------------------------------------------------------------

float getTemp(){ // fonction permettant de lire la temperature en degres celsius

  byte data[12];
  byte addr[8];

  if ( !ds.search(addr)) {
      //no more sensors on chain, reset search
      ds.reset_search();
      return -1000;
  }

  if ( OneWire::crc8( addr, 7) != addr[7]) {
      Serial.println("CRC is not valid!");
      return -1000;
  }

  if ( addr[0] != 0x10 && addr[0] != 0x28) {
      Serial.print("Device is not recognized");
      return -1000;
  }

  ds.reset();
  ds.select(addr);
  ds.write(0x44,1); 

  byte present = ds.reset();
  ds.select(addr);
  ds.write(0xBE); 

  for (int i = 0; i < 9; i++) { 
    data[i] = ds.read();
  }

  ds.reset_search();

  byte MSB = data[1];
  byte LSB = data[0];

  float tempRead = ((MSB << 8) | LSB); 
  float TemperatureSum = tempRead / 16;

  return TemperatureSum;
}



//--------------------------------------------------------------------------------------------------
//-------------------------------------Fonction Mesure pH-------------------------------------------
//--------------------------------------------------------------------------------------------------

float getpH(){ // fonction permettant de mesurer le pH

  unsigned long int avg_pH;  // valeur moyenne de la mesure
  int buf[10],k;

  for(int i=0;i<10;i++)       // fait 10 mesures afin d'obtenir une valeur moyenne
  { 
    buf[i]=analogRead(pin_pH);
    delay(10);
  }
  for(int i=0;i<9;i++)        // classe les valeurs
  {
    for(int j=i+1;j<10;j++)
    {
      if(buf[i]>buf[j])
      {
        k=buf[i];
        buf[i]=buf[j];
        buf[j]=k;
     }
    }
  }
  
  avg_pH=0;
  for(int i=2;i<8;i++)      // calcule la valeur moyenne des 6 mesures centrales
    avg_pH+=buf[i];
    
  float V = (float)avg_pH * (float)VREF / 1024.0 / 6.0; // convertit la mesure en volts
  float phValue = 0.00247 * V + 4.02;  // convertit la tension en pH (d'après la fonction de calibrage de la sonde)
 
  return phValue;
}



//--------------------------------------------------------------------------------------------------
//-------------------------------------Fonction Mesure EC-------------------------------------------
//--------------------------------------------------------------------------------------------------

float getEC(float temperature){ // fonction permettant de mesurer EC  

  unsigned long int avg_EC;  
  int buf[10],k;

  for(int i=0;i<10;i++)      
  { 
    buf[i]=analogRead(pin_EC);
    delay(10);
  }
  for(int i=0;i<9;i++)        
  {
    for(int j=i+1;j<10;j++)
    {
      if(buf[i]>buf[j])
      {
        k=buf[i];
        buf[i]=buf[j];
        buf[j]=k;
     }
    }
  }
  
  avg_EC=0;
  for(int i=2;i<8;i++)      
    avg_EC+=buf[i];
    
  float averageVoltage = (float)avg_EC * (float)VREF / 1024.0 / 6.0; 
  float compensationCoefficient=1.0+0.0185*(temperature-25.0);    // calcule le coeff de compensation en temperature
  float compensationVoltage=averageVoltage/compensationCoefficient;  // applique la compensation en temperature
  float ECValue=4.00 * compensationVoltage - 0.696;
 
  return ECValue;
}




//--------------------------------------------------------------------------------------------------
//-------------------------------------Fonction Mesure Turbidite------------------------------------
//--------------------------------------------------------------------------------------------------

float getNTU(){ // fonction permettant de mesurer la turbidite

  unsigned long int avg_NTU;  
  int buf[10],k;

  for(int i=0;i<10;i++)      
  { 
    buf[i]=analogRead(pin_NTU);
    delay(10);
  }
  for(int i=0;i<9;i++)       
  {
    for(int j=i+1;j<10;j++)
    {
      if(buf[i]>buf[j])
      {
        k=buf[i];
        buf[i]=buf[j];
        buf[j]=k;
     }
    }
  }
  
  avg_NTU=0;
  for(int i=2;i<8;i++)     
    avg_NTU+=buf[i];
    
  float V = (float)avg_NTU * (float)VREF / 1024.0 / 6.0; 
  float NTUValue = 8030 - 11.3 * V - 0.00376 * V * V;  
  
  return NTUValue;
}



//--------------------------------------------------------------------------------------------------
//-------------------------------------Fonction Mesure Oxygene Dissous------------------------------
//--------------------------------------------------------------------------------------------------

float getDO(float temperature){ // fonction permettant de mesurer la concentration en oxygene dissous

  unsigned long int avg_DO;  
  int buf[10],k;

  for(int i=0;i<10;i++)      
  { 
    buf[i]=analogRead(pin_DO);
    delay(10);
  }
  for(int i=0;i<9;i++)        
  {
    for(int j=i+1;j<10;j++)
    {
      if(buf[i]>buf[j])
      {
        k=buf[i];
        buf[i]=buf[j];
        buf[j]=k;
     }
    }
  }
  
  avg_DO=0;
  for(int i=2;i<8;i++)      
    avg_DO+=buf[i];
    
  float V = (float)avg_DO * (float)VREF / 1024.0 / 6.0; 
  float DOValue= (3.869 + 10.65 * exp(- temperature / 27.99)) / (26.2 * temperature + 643) * V;  
 
  return DOValue;
}



//--------------------------------------------------------------------------------------------------
//-----------------------------------------Horloge RTC----------------------------------------------
//--------------------------------------------------------------------------------------------------

void setTimeFromCompile() { // donne l'heure depuis la compilation
  
  String compileTime = String(__TIME__);
  
  int h = compileTime.substring(0, 2).toInt();
  int m = compileTime.substring(3, 5).toInt();
  int s = compileTime.substring(6, 8).toInt();
  
  rtc.setTime(h, m, s);
}

void setDateFromCompile() { // donne la date depuis la compilation
  String months[] = {
    "Jan", "Feb", "Mar", "Apr", "May", "Jun",
    "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
  };
 
  String compileDate = String(__DATE__);

  String monthStr = compileDate.substring(0, 3);

  int m = 0;    

  for (int i = 0; i < 12; i++) {
    if (monthStr == months[i]) {
      m = i + 1;
      break;
    }
  }
  
  int d = compileDate.substring(4, 6).toInt();
  int y = compileDate.substring(9, 11).toInt();
 
  rtc.setDate(d, m, y);
}

void alarmMatch(){ // passe le booleen "matched" en "true"
  matched=true;
}
